unit PainterReduceU;
////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// Project:	TGIFImage demo application.                                   //
// Description:	Demonstration of bitmap import with automatic color reduction.//
// Copyright	(c) 1997,98 Anders Melander.                                  //
//		All rights reserved.                                          //
// Formatting:	2 space indent, 8 space tabs, 80 columns.                     //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// To make this demo work with Delphi 2 or C++ builder, you will need to      //
// load a bitmap (with *many* colors preferably) into the ImageBitmap         //
// TImage component on the form.                                              //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////
interface

uses
  gifimage,
{$IFNDEF VER90}
{$IFNDEF BCB}
  // Sorry guys - No jpeg in Delphi 2 and C++ Builder 3
  jpeg,
{$ENDIF}
{$ENDIF}
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls, Menus, Buttons;

type
  TPainterReduceF = class(TForm)
    ProgressBar: TProgressBar;
    PanelSource: TPanel;
    Label1: TLabel;
    Panel2: TPanel;
    PanelDestination: TPanel;
    Label2: TLabel;
    MainMenu1: TMainMenu;
    MenuPalette: TMenuItem;
    MenuDithering: TMenuItem;
    MenuDitheringFloydSteinberg: TMenuItem;
    MenuDitheringNearest: TMenuItem;
    MenuPaletteNone: TMenuItem;
    MenuPaletteWindows20: TMenuItem;
    MenuPaletteWindows256: TMenuItem;
    MenuPaletteWindowsGray4: TMenuItem;
    MenuPaletteMonochrome: TMenuItem;
    MenuPaletteGrayScale256: TMenuItem;
    MenuPaletteNetscape216: TMenuItem;
    MenuPaletteOptimized: TMenuItem;
    MenuPaletteOptimized8: TMenuItem;
    MenuPaletteOptimized16: TMenuItem;
    MenuPaletteOptimized32: TMenuItem;
    MenuPaletteOptimized64: TMenuItem;
    MenuPaletteOptimized128: TMenuItem;
    MenuPaletteOptimized256: TMenuItem;
    MenuPaletteOptimizedWindows256: TMenuItem;
    ScrollBox1: TScrollBox;
    ImageBitmap: TImage;
    ScrollBox2: TScrollBox;
    ImageGIF: TImage;
    ButtonConvert: TSpeedButton;
    sizecap: TLabel;
    MenuDitheringStucki: TMenuItem;
    MenuDitheringBurkes: TMenuItem;
    MenuDitheringSierra: TMenuItem;
    MenuDitheringJaJuNi: TMenuItem;
    MenuDitheringSteveArche: TMenuItem;
    BitBtn1: TBitBtn;
    BitBtn2: TBitBtn;
    procedure ButtonConvertClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure MenuDitheringClick(Sender: TObject);
    procedure MenuPaletteOptimizedBitsClick(Sender: TObject);
    procedure MenuPaletteClick(Sender: TObject);
    procedure MenuPaletteHalftoneClick(Sender: TObject);
    procedure btnsaveClick(Sender: TObject);
  private
    { Private declarations }
    procedure OnProgress(Sender: TObject; Stage: TProgressStage;
      PercentDone: Byte; RedrawNow: Boolean; const R: TRect; const Msg: string);
  public
    { Public declarations }
    procedure ExternalConvert(bitmap: Tbitmap);    
  end;

var
  PainterReduceF: TPainterReduceF;
  GIFrm: TColorReduction;
  GIFdm: TDitherMode;
implementation

{$R *.DFM}

procedure TPainterReduceF.ExternalConvert(bitmap:Tbitmap);
var
  GIF			: TGIFImage;
  ms:tmemorystream;
begin
  try
    // Set color reduction options
      // Set import color reduction options
      if (MenuPaletteNetscape216.Checked) then
        GIFrm:= rmNetscape
      else if (MenuPaletteMonochrome.Checked) then
        GIFrm := rmMonochrome
      else if (MenuPaletteGrayScale256.Checked) then
        GIFrm := rmGrayScale
      else if (MenuPaletteWindowsGray4.Checked) then
        GIFrm := rmWindowsGray
      else if (MenuPaletteWindows20.Checked) then
        GIFrm := rmWindows20
      else if (MenuPaletteWindows256.Checked) then
        GIFrm := rmWindows256
      else if (MenuPaletteOptimized.Checked) then
        GIFrm := rmQuantize
      else if (MenuPaletteOptimizedWindows256.Checked) then
        GIFrm := rmQuantizeWindows
      else
        GIFrm := rmNone;
       GIFImageDefaultColorReduction:=GIFrm;
    if (MenuDitheringFloydSteinberg.Checked) then
      GIFdm := dmFloydSteinberg
    else if (MenuDitheringStucki.Checked) then
      GIFdm := dmStucki
    else if (MenuDitheringSierra.Checked) then
      GIFdm := dmSierra
    else if (MenuDitheringJaJuNi.Checked) then
      GIFdm := dmJaJuNi
    else if (MenuDitheringSteveArche.Checked) then
      GIFdm := dmSteveArche
    else if (MenuDitheringBurkes.Checked) then
      GIFdm := dmBurkes
    else if (MenuDitheringNearest.Checked) then
      GIFdm := dmNearest;
    GIFImageDefaultDitherMode:=GIFdm;
    // Import bitmap into a GIF
    GIF := TGIFImage.Create;
    try
      GIF.OnProgress := OnProgress;
      GIF.Assign(bitmap);
      bitmap.Assign(GIF);
    finally
      GIF.Free;

    end;
  finally
//
  end;
end;

procedure TPainterReduceF.ButtonConvertClick(Sender: TObject);
var
  GIF			: TGIFImage;
  ms:tmemorystream;
begin
  // Display warning for Delphi 2
{$ifdef VER90}
  if (PaletteDevice) then
    ShowMessage('Color reduction can not be used with Delphi 2 in 256 color mode!');
{$endif}
  if (ImageBitmap.Picture.Graphic = nil) then
    ShowMessage('No bitmap to convert!');
  ButtonConvert.Enabled := False;
  try
    // Clear previous GIF view
    ImageGIF.Picture.Assign(nil);

    // Set color reduction options
      // Set import color reduction options
      if (MenuPaletteNetscape216.Checked) then
        GIFImageDefaultColorReduction := rmNetscape
      else if (MenuPaletteMonochrome.Checked) then
        GIFImageDefaultColorReduction := rmMonochrome
      else if (MenuPaletteGrayScale256.Checked) then
        GIFImageDefaultColorReduction := rmGrayScale
      else if (MenuPaletteWindowsGray4.Checked) then
        GIFImageDefaultColorReduction := rmWindowsGray
      else if (MenuPaletteWindows20.Checked) then
        GIFImageDefaultColorReduction := rmWindows20
      else if (MenuPaletteWindows256.Checked) then
        GIFImageDefaultColorReduction := rmWindows256
      else if (MenuPaletteOptimized.Checked) then
        GIFImageDefaultColorReduction := rmQuantize
      else if (MenuPaletteOptimizedWindows256.Checked) then
        GIFImageDefaultColorReduction := rmQuantizeWindows
      else
        GIFImageDefaultColorReduction := rmNone;

    if (MenuDitheringFloydSteinberg.Checked) then
      GIFImageDefaultDitherMode := dmFloydSteinberg
    else if (MenuDitheringStucki.Checked) then
      GIFImageDefaultDitherMode := dmStucki
    else if (MenuDitheringSierra.Checked) then
      GIFImageDefaultDitherMode := dmSierra
    else if (MenuDitheringJaJuNi.Checked) then
      GIFImageDefaultDitherMode := dmJaJuNi
    else if (MenuDitheringSteveArche.Checked) then
      GIFImageDefaultDitherMode := dmSteveArche
    else if (MenuDitheringBurkes.Checked) then
      GIFImageDefaultDitherMode := dmBurkes
    else if (MenuDitheringNearest.Checked) then
      GIFImageDefaultDitherMode := dmNearest;

    // Import bitmap into a GIF
    GIF := TGIFImage.Create;
    try
      GIF.OnProgress := OnProgress;
      GIF.Assign(ImageBitmap.Picture.Graphic);
      ms:=tmemorystream.Create ;
      Gif.SaveToStream (ms);
      sizecap.Caption :='Size= '+inttostr(ms.Size)+' bytes' ;
      ms.free;
      ImageGIF.Picture.Assign(GIF);
    finally
      GIF.Free;

    end;
  finally
    ButtonConvert.Enabled := True;
  end;
end;

procedure TPainterReduceF.OnProgress(Sender: TObject; Stage: TProgressStage;
  PercentDone: Byte; RedrawNow: Boolean; const R: TRect; const Msg: string);
begin
  if (Stage= psEnding) then
    ProgressBar.Position := 0
  else
    ProgressBar.Position := PercentDone;
end;

procedure TPainterReduceF.FormResize(Sender: TObject);
begin
  PanelSource.Width := ClientWidth DIV 2;
end;

procedure TPainterReduceF.MenuDitheringClick(Sender: TObject);
begin
  TMenuItem(Sender).Checked := True;
end;

procedure TPainterReduceF.MenuPaletteOptimizedBitsClick(Sender: TObject);
begin
  GIFImageDefaultColorReductionBits := TMenuItem(Sender).Tag;
  TMenuItem(Sender).Checked := True;
end;

procedure TPainterReduceF.MenuPaletteClick(Sender: TObject);
begin
  TMenuItem(Sender).Checked := True;
end;

procedure TPainterReduceF.MenuPaletteHalftoneClick(Sender: TObject);
var
  DesktopDC		: HDC;
  BitsPerPixel		: integer;
begin
  DesktopDC := GetDC(0);
  try
    BitsPerPixel := GetDeviceCaps(DesktopDC, BITSPIXEL) * GetDeviceCaps(DesktopDC, PLANES);
  finally
    ReleaseDC(0, DesktopDC);
  end;
  if (BitsPerPixel > 8) then
    ShowMessage('Please note that the Windows Halftone color reduction method'+#13+
                'only works properly in 256 color display mode.'+#13+#13+
                'Since your display adapter is currently in '+IntToStr(1 SHL BitsPerPixel)+' color mode,'+#13+
                'the Windows Halftone Palette method will produce the same'+#13+
                'result as the Windows System Palette method.');

  MenuPaletteClick(Sender);
end;

procedure TPainterReduceF.btnsaveClick(Sender: TObject);

begin
end;
{
procedure TFormAnimate.ButtonAnimateClick(Sender: TObject);
var
  GIF			: TGIFImage;
  i			: integer;

  function TransparentIndex(GIF: TGIFSubImage): byte;
  begin
    // Use the lower left pixel as the transparent color
    Result := GIF.Pixels[0, GIF.Height-1];
  end;

  function AddBitmap(GIF: TGIFImage; Source: TGraphic; Transparent: boolean): integer;
  var
    Ext			: TGIFGraphicControlExtension;
    LoopExt		: TGIFAppExtNSLoop;
  begin
    // Preview the image being added (for user feedback)
    ImageAnimate.Picture.Assign(Source);
    ImageAnimate.Update;

    // Add the source image to the animation
    Result := GIF.Add(Source);
    // Netscape Loop extension must be the first extension in the first frame!
    if (Result = 0) then
    begin
      LoopExt := TGIFAppExtNSLoop.Create(GIF.Images[Result]);
      LoopExt.Loops := 0; // Number of loops (0 = forever)
      GIF.Images[Result].Extensions.Add(LoopExt);
    end;
    // Add Graphic Control Extension
    Ext := TGIFGraphicControlExtension.Create(GIF.Images[Result]);
    Ext.Delay := 30; // Animation delay (30 = 300 mS)
    if (Transparent) then
    begin
      Ext.Transparent := True;
      Ext.TransparentColorIndex := TransparentIndex(GIF.Images[Result]);
    end;
    GIF.Images[Result].Extensions.Add(Ext);
  end;

begin
  Screen.Cursor := crHourGlass;
  try
    GIF := TGIFImage.Create;
    try
      // Add all the images in the scroll box to the animation
      for i := 0 to ScrollBoxSource.ControlCount-1 do
        // Add frame
        AddBitmap(GIF, // Destination GIF
          TImage(ScrollBoxSource.Controls[i]).Picture.Graphic, // Source bitmap
          (i > 0)); // First frame is not transparent

      // Create a single global color map from multiple local color maps.
      // You don't *need* to do this, but it reduces the size of the file
      GIF.OptimizeColorMap;

      // Display the animation
      ImageAnimate.Picture.Assign(GIF);
    finally
      GIF.Free;
    end;
    ButtonSave.Enabled := True;
  finally
    Screen.Cursor := crDefault;
  end;
end;}


end.
